import datetime

from cpl_core.configuration import ConfigurationABC

from bot_data.abc.user_joined_game_server_repository_abc import UserJoinedGameServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_message_count_per_hour_repository_abc import UserMessageCountPerHourRepositoryABC
from bot_data.model.server_config import ServerConfig
from bot_graphql.abc.query_abc import QueryABC


class ServerStatisticQuery(QueryABC):
    def __init__(
        self,
        config: ConfigurationABC,
        user_joined_voice_channels: UserJoinedVoiceChannelRepositoryABC,
        user_joined_game_servers: UserJoinedGameServerRepositoryABC,
        user_messages: UserMessageCountPerHourRepositoryABC,
    ):
        QueryABC.__init__(self, "ServerStatistic")

        self._config = config

        self._user_joined_voice_channels = user_joined_voice_channels
        self._user_joined_game_servers = user_joined_game_servers
        self._user_messages = user_messages

        self.add_field("userJoinedVoiceChannelCount", self._resolve_voice_channel_count)
        self.add_field("userJoinedVoiceChannelOntime", self._resolve_voice_channel_ontime)
        self.add_field("userJoinedGameServerCount", self._resolve_game_server_count)
        self.add_field("userJoinedGameServerOntime", self._resolve_game_server_ontime)
        self.add_field(
            "messageCount",
            self._resolve_message_count,
        )

    def _resolve_voice_channel_count(self, server, *_, **kwargs):
        return (
            self._user_joined_voice_channels.get_user_joined_voice_channels()
            .where(lambda x: x.user.server.id == server.id and x.created_at.date() >= self._get_date(**kwargs))
            .count()
        )

    def _resolve_voice_channel_ontime(self, server, *_, **kwargs):
        return (
            self._user_joined_voice_channels.get_user_joined_voice_channels()
            .where(lambda x: x.user.server.id == server.id and x.created_at.date() >= self._get_date(**kwargs))
            .sum(lambda x: x.time)
        )

    def _resolve_game_server_count(self, server, *_, **kwargs):
        return (
            self._user_joined_game_servers.get_user_joined_game_servers()
            .where(lambda x: x.user.server.id == server.id and x.created_at.date() >= self._get_date(**kwargs))
            .count()
        )

    def _resolve_game_server_ontime(self, server, *_, **kwargs):
        return (
            self._user_joined_game_servers.get_user_joined_game_servers()
            .where(lambda x: x.user.server.id == server.id and x.created_at.date() >= self._get_date(**kwargs))
            .sum(lambda x: x.time)
        )

    def _resolve_message_count(self, server, *_, **kwargs):
        settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{server.discord_id}")
        return (
            self._user_messages.get_user_message_count_per_hours()
            .where(lambda x: x.user.server.id == server.id and x.created_at.date() >= self._get_date(**kwargs))
            .sum(lambda x: x.xp_count / settings.xp_per_message)
        )

    def _get_date(self, **kwargs) -> datetime.date:
        if "date" not in kwargs:
            return datetime.date.today() - datetime.timedelta(days=7)

        return datetime.datetime.strptime(kwargs["date"], "%d.%m.%Y").date()
